<?php

namespace Drupal\reset_password_email_otp\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure reset password email otp settings.
 */
class ResetPasswordMailOTPConfig extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'reset_password_email_otp.settings';

  /**
   * Helper to get the config keys.
   *
   * @return string[]
   *   The configuration keys.
   */
  public static function getConfigKeys(): array {
    return [
      'reset_password_email_otp_wrong_attempt',
      'reset_password_email_otp_length',
      'reset_password_email_otp_mail_subject',
      'reset_password_email_otp_mail_body',
      'step_one_title',
      'step_one_email_or_username_label',
      'step_one_email_or_username_description',
      'step_one_next',
      'step_two_title',
      'step_two_otp_label',
      'step_two_otp_description',
      'step_two_back',
      'step_two_next',
      'step_three_title',
      'step_three_password_description',
      'step_three_back',
      'step_three_submit',
      'step_four_login',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'reset_password_email_otp_config';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['reset_password_email_otp_wrong_attempt'] = [
      '#title' => $this->t('Wrong attempt OTP limit'),
      '#type' => 'textfield',
      '#description' => $this->t('How many time user can attempt wrong OTP.'),
      '#default_value' => $config->get('reset_password_email_otp_wrong_attempt'),
    ];
    $form['reset_password_email_otp_length'] = [
      '#title' => $this->t('Generate OTP character length'),
      '#type' => 'textfield',
      '#description' => $this->t('Generate OTP character length.'),
      '#default_value' => $config->get('reset_password_email_otp_length'),
    ];

    $form['step_one'] = [
      '#type' => 'details',
      '#title' => $this->t('Step one'),
    ];
    $form['step_one']['step_one_title'] = [
      '#title' => $this->t('Title'),
      '#type' => 'textfield',
      '#description' => $this->t('This is the heading 2 appearing above the form step 1.'),
      '#default_value' => $config->get('step_one_title'),
    ];
    $form['step_one']['step_one_email_or_username_label'] = [
      '#title' => $this->t('Email or username field label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_one_email_or_username_label'),
      '#required' => TRUE,
    ];
    $form['step_one']['step_one_email_or_username_description'] = [
      '#title' => $this->t('Email or username field description'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_one_email_or_username_description'),
    ];
    $form['step_one']['step_one_next'] = [
      '#title' => $this->t('Next button label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_one_next'),
      '#required' => TRUE,
    ];

    $form['step_two'] = [
      '#type' => 'details',
      '#title' => $this->t('Step two'),
    ];
    $form['step_two']['step_two_title'] = [
      '#title' => $this->t('Title'),
      '#type' => 'textfield',
      '#description' => $this->t('This is the heading 2 appearing above the form step 2.'),
      '#default_value' => $config->get('step_two_title'),
    ];
    $form['step_two']['step_two_otp_label'] = [
      '#title' => $this->t('One time passcode label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_two_otp_label'),
      '#required' => TRUE,
    ];
    $form['step_two']['step_two_otp_description'] = [
      '#title' => $this->t('One time passcode field description'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_two_otp_description'),
    ];
    $form['step_two']['step_two_back'] = [
      '#title' => $this->t('Back button label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_two_back'),
      '#required' => TRUE,
    ];
    $form['step_two']['step_two_next'] = [
      '#title' => $this->t('Next button label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_two_next'),
      '#required' => TRUE,
    ];

    $form['step_three'] = [
      '#type' => 'details',
      '#title' => $this->t('Step three'),
    ];
    $form['step_three']['step_three_title'] = [
      '#title' => $this->t('Title'),
      '#type' => 'textfield',
      '#description' => $this->t('This is the heading 2 appearing above the form step 3.'),
      '#default_value' => $config->get('step_three_title'),
    ];
    $form['step_three']['step_three_password_description'] = [
      '#title' => $this->t('Password field description'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_three_password_description'),
    ];
    $form['step_three']['step_three_back'] = [
      '#title' => $this->t('Back button label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_three_back'),
      '#required' => TRUE,
    ];
    $form['step_three']['step_three_submit'] = [
      '#title' => $this->t('Submit button label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_three_submit'),
      '#required' => TRUE,
    ];

    $form['step_four'] = [
      '#type' => 'details',
      '#title' => $this->t('Step four'),
    ];
    $form['step_four']['step_four_login'] = [
      '#title' => $this->t('Login button label'),
      '#type' => 'textfield',
      '#default_value' => $config->get('step_four_login'),
      '#required' => TRUE,
    ];

    $form['email'] = [
      '#type' => 'details',
      '#title' => $this->t('Email'),
    ];
    $form['email']['reset_password_email_otp_mail_subject'] = [
      '#title' => $this->t('Reset Mail OTP mail subject'),
      '#type' => 'textfield',
      '#description' => $this->t("Reset Password Mail OTP mail subject content.<br>The following tokens are supported: [user:display-name], [user:account-name], [user:mail]"),
      '#default_value' => $config->get('reset_password_email_otp_mail_subject'),
    ];
    $form['email']['reset_password_email_otp_mail_body'] = [
      '#title' => $this->t('Reset Password Mail OTP mail body'),
      '#type' => 'textarea',
      '#description' => $this->t("Make sure to use [OTP] in email body for contains OTP value.<br>The following tokens are supported: [user:display-name], [user:account-name], [user:mail]"),
      '#default_value' => $config->get('reset_password_email_otp_mail_body'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $otp = $form_state->getValue('reset_password_email_otp_mail_body');
    $pattern = "/\[OTP\]/";
    if (!preg_match($pattern, $otp)) {
      $form_state->setErrorByName('reset_password_email_otp_mail_body', $this->t('Please add [OTP] text in Email body.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('reset_password_email_otp.settings');
    // Save form values.
    foreach ($this->getConfigKeys() as $key) {
      $config->set($key, $form_state->getValue($key))->save();
    }

    Cache::invalidateTags(['reset_password_email_otp']);

    parent::submitForm($form, $form_state);
  }

}
